package com.example.fuzzControll.service.impl;

import com.alibaba.fastjson.JSON;
import com.example.fuzzControll.constents.MissionStateEnum;
import com.example.fuzzControll.constents.TableClassEnum;
import com.example.fuzzControll.domain.bo.CmdStartParams;
import com.example.fuzzControll.domain.bo.FuzzParams;
import com.example.fuzzControll.domain.bo.TestEntity;
import com.example.fuzzControll.domain.po.MissionInfo;
import com.example.fuzzControll.domain.po.Test;
import com.example.fuzzControll.domain.po.TestAndParams;
import com.example.fuzzControll.exception.mysqlException.MysqlException;
import com.example.fuzzControll.mapper.FuzzParamsMapper;
import com.example.fuzzControll.mapper.MissionInfoMapper;
import com.example.fuzzControll.mapper.TestMapper;
import com.example.fuzzControll.service.TestMissionService;
import com.example.fuzzControll.tools.system.GlobalClass;
import com.example.fuzzControll.tools.system.SystemRunningParams;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Slf4j
@Service("TestMissionServiceImpl")
public class TestMissionServiceImpl implements TestMissionService {
    @Autowired
    TestMapper testMapper;
    @Autowired
    FuzzParamsMapper fuzzParamsMapper;
    @Autowired
    MissionInfoMapper missionInfoMapper;

    @Override
    public List<TestAndParams> getTestAndParamsList() {
        List<TestAndParams> testParamsRelevantList = new ArrayList();
        List<TestAndParams> testParamsListEmpty = new ArrayList();
        try {
            /*查询关联的数据*/
            testParamsRelevantList = testMapper.getTestAndParamsList();
            /*查询空白数据，组合起来是全数据*/
            testParamsListEmpty = testMapper.getTestList().stream().map(test -> new TestAndParams(test.getId(), test.getFuzzParamsId(), test.getCreateTime()
                    , test.getUpdateTime(), test.getMissionIds(), test.getTestName(), "", "")).collect(Collectors.toList());
            testParamsRelevantList.addAll(testParamsListEmpty);
        } catch (Exception e) {
            log.error("Failed to get test and params list:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        if (testParamsRelevantList == null || testParamsRelevantList.size() == 0) {
            log.warn("TestAndParams is null!");
        }
        return testParamsRelevantList;
    }

    @Override
    public boolean deleteById(int id) {
        boolean flag = false;
        try {
            Test test = testMapper.getTestById(id);
            /*非空白测试级联删除*/
            if (!(test.getFuzzParamsId() == 0)) {
                flag = testMapper.deleteById(id);
            } else {
                /*空白测试删除*/
                flag = testMapper.deleteEmptyById(id);
            }
        } catch (Exception e) {
            log.error("Delete error:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        return flag;
    }


    @Override
    public void insert(TestAndParams testAndParams) {
        Test test = new Test(new Date(), new Date(), testAndParams.getTestName());
        testMapper.insert(test);
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestAflnet(int id, CmdStartParams cmdStartParams) {
        try {
            /*判断test是否存有paramsId,没数据插入有数据就改变*/
            Test test = testMapper.getTestById(id);
            int fuzzParamsId;
            if (test.getFuzzParamsId() == 0) {//为0则插入id
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                fuzzParamsMapper.insertFuzzParamsReturnId(fuzzParams);
                fuzzParamsId = fuzzParams.getId();
                test.setFuzzParamsId(fuzzParamsId);
            } else {//有则更新数据
                FuzzParams fuzzParams = new FuzzParams(test.getFuzzParamsId(), JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                if (!fuzzParamsMapper.editFuzzParams(fuzzParams.getId(), fuzzParams.getParams(), fuzzParams.getProtocol())) {
                    throw new RuntimeException("EditFuzzParams failed!");
                }
            }
            /*更新修改时间*/
            test.setUpdateTime(new Date());
            testMapper.edit(test);
        } catch (Exception e) {
            log.error("Failed to insert:[{}]", e.getMessage());
            throw new MysqlException("");
        }
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestKitty(int id, TestEntity testEntity) {
        try {
            /*判断test是否存有paramsId,没数据插入有数据就改变*/
            Test test = testMapper.getTestById(id);
            int fuzzParamsId;
            if (test.getFuzzParamsId() == 0) {//为0则插入id
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(testEntity.getParamJson()), testEntity.getTestClassName());
                fuzzParamsMapper.insertFuzzParamsReturnId(fuzzParams);
                fuzzParamsId = fuzzParams.getId();
                test.setFuzzParamsId(fuzzParamsId);
            } else {//有则更新数据
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(testEntity.getParamJson()), testEntity.getTestClassName());
                if (!fuzzParamsMapper.editFuzzParams(fuzzParams.getId(), fuzzParams.getParams(), fuzzParams.getProtocol())) {
                    throw new RuntimeException("EditFuzzParams failed!");
                }
            }
            /*更新修改时间*/
            test.setUpdateTime(new Date());
            testMapper.edit(test);
        } catch (Exception e) {
            log.error("Failed to insert:[{}]", e.getMessage());
            throw new MysqlException("");
        }
    }

    @Override
    public boolean editTestName(String testName, int id) {
        boolean flag = false;
        try {
            flag = testMapper.editTestName(testName, id);
        } catch (Exception e) {
            log.error("EditTestName failed!:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
        return flag;
    }

    @Override
    public void startMission(int id) {
        Test test = testMapper.getTestById(id);
        if (!(test.getFuzzParamsId() == 0)) {
            log.error("This is a empty test!");
            throw new RuntimeException();
        }
        /*生成任务*/
        int missionId = GlobalClass.missionInfoMapper.selectTopMissionId() + 1;
        SystemRunningParams.kittyMissionId = missionId;
        MissionInfo missionInfo = new MissionInfo(missionId, TableClassEnum.KITTY_RESULT.getTableId(), new Date(), SystemRunningParams.kittyData.get("missionName"),
                MissionStateEnum.RUNNING.getStateCode(), 0L);

    }
}
